﻿using System;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Xml.Serialization;
using Newtonsoft.Json;
using ODM.Backend.Models.XmlSettings;
using Skeleton.Mobile.Framework.Data.Http.Files;
using Skeleton.Mobile.Framework.Error.Log;
using Skeleton.Mobile.ODM.Backend.Configuration;

namespace Skeleton.Mobile.ODM.Backend.DataAccess.WebService
{
	/// <summary>
	/// Třída se stará o requesty na webovou službu ODM, adresa na kterou se odkazuje se nachází v ODMConfig
	/// </summary>
	public class ODMWebRepository
	{
		public static Func<int, string> StaticDataAction = (int version) => $"data/static/{version.ToString()}";
		public static Func<int, string> MessageDataAction = (int version) => $"data/message/{version.ToString()}";
		public static Func<int, string> ProgramDataAction = (int version) => $"data/program/{version.ToString()}";
		public static Func<int, string> ResultsDataAction = (int version) => $"data/result/{version.ToString()}";
		public static string StateDataAction => "data/state/";
		
		public async Task<FileState> CheckDataState()
		{
			var webRepository = new FileWebRepository("");
			var serviceUrl = new ODMConfigurationFactory().AppWebServiceConfiguration.SelectedWebService.URL;

			byte[] data = null;
			FileState result = new FileState();
			try
			{
				data = await webRepository.GetFileFromUrl(
					serviceUrl + StateDataAction,
					new CancellationToken(), 5000);
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR,
					$"State data check - exception: {ex}, ODMWebRepository.CheckDataState");
			}

			try
			{
				if (data != null)
				{
					string response;
					using (var stream = new MemoryStream(data))
					using(var streamreader = new StreamReader(stream))
					{
						response = await streamreader.ReadToEndAsync();
					}

					var serializer = new JsonSerializer();
					using (TextReader reader = new StringReader(response))
					{
						var jsonreader = new JsonTextReader(reader);
						result = serializer.Deserialize<FileState>(jsonreader);
					}
				}
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR,
					$"Service returned unexpected response, Data not checked, ex: {ex}, ODMWebRepository.CheckDataState");
			}

			return result;
		}

		public async Task<byte[]> GetStaticData(int localVersion)
		{
			var webRepository = new FileWebRepository("");
			var serviceUrl = new ODMConfigurationFactory().AppWebServiceConfiguration.SelectedWebService.URL;

			byte[] output = null;

			try
			{
				output = await webRepository.GetFileFromUrl(serviceUrl + StaticDataAction(localVersion), new CancellationToken());
				if (output == null)
				{
					await ErrorLog.Instance.AddRecord(ErrorTypes.WARNING,
						$"Data se nestahla - ODMWebRepository.GetStaticData");
				}
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, $"Chyba pri stahovani dat - ex: {ex}");
			}

			return output;
		}
 
		public async Task<byte[]> GetMessageData(int localVersion)
		{
			var webRepository = new FileWebRepository("");
			var serviceUrl = new ODMConfigurationFactory().AppWebServiceConfiguration.SelectedWebService.URL;

			byte[] output = null;

			try
			{
				output = await webRepository.GetFileFromUrl(serviceUrl + MessageDataAction(localVersion), new CancellationToken());
				if (output == null)
				{
					await ErrorLog.Instance.AddRecord(ErrorTypes.WARNING,
						$"Data se nestahla - ODMWebRepository.GetMessageData");
				}
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, $"Chyba pri stahovani dat - ex: {ex}");
			}

			return output;
		}

		public async Task<byte[]> GetProgramData(int localVersion)
		{
			var webRepository = new FileWebRepository("");
			var serviceUrl = new ODMConfigurationFactory().AppWebServiceConfiguration.SelectedWebService.URL;

			byte[] output = null;

			try
			{
				output = await webRepository.GetFileFromUrl(serviceUrl + ProgramDataAction(localVersion), new CancellationToken());
				if (output == null)
				{
					await ErrorLog.Instance.AddRecord(ErrorTypes.WARNING,
						$"Data se nestahla - ODMWebRepository.GetProgramData");
				}
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, $"Chyba pri stahovani dat - ex: {ex}");
			}

			return output;
		}

		public async Task<byte[]> GetResultsData(int localVersion)
		{
			var webRepository = new FileWebRepository("");
			var serviceUrl = new ODMConfigurationFactory().AppWebServiceConfiguration.SelectedWebService.URL;

			byte[] output = null;

			try
			{
				output = await webRepository.GetFileFromUrl(serviceUrl + ResultsDataAction(localVersion), new CancellationToken());
				if (output == null)
				{
					await ErrorLog.Instance.AddRecord(ErrorTypes.WARNING,
						$"Data se nestahla - ODMWebRepository.GetResultsData");
				}
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, $"Chyba pri stahovani dat - ex: {ex}");
			}

			return output;
		}
	}
}