﻿using System;
using System.IO;
using System.Runtime.CompilerServices;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Xml.Serialization;
using Newtonsoft.Json;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Device;
using Skeleton.Mobile.Framework.Error.Log;
using Skeleton.Mobile.ODM.Backend.DataAccess.InternalStorage;
using FileInfo = ODM.Backend.Models.XmlSettings.FileInfo;

namespace Skeleton.Mobile.ODM.Backend.Managers
{
	/// <summary>
	/// Třída se stará o uložení dat do interního uložiště a čtení těchto dat.
	/// </summary>
	public static class DataManager
	{
		private static string FileExtension => ".json";

		#region DataGroups

		private static StaticAppData staticAppData;
		private static MessageAppData messageAppData;
		private static ProgramAppData programAppData;
		private static ResultAppData resultAppData;

		#endregion
		//Semafory zabraňují paralelnímu přístupu do souborů v uložišti
		private static SemaphoreSlim semaphoreStaticData = new SemaphoreSlim(1);
		private static SemaphoreSlim semaphoreMessageData = new SemaphoreSlim(1);
		private static SemaphoreSlim semaphoreProgramData = new SemaphoreSlim(1);
		private static SemaphoreSlim semaphoreResultsData = new SemaphoreSlim(1);

		internal static async Task<StaticAppData> GetStaticData()
		{
			StaticAppData output = null;
			try
			{
				if (staticAppData == null)
				{
					await semaphoreStaticData.WaitAsync();
					staticAppData = await LoadStaticData();
					await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, $"Loading static data from path - {FileInfo.StaticDataFileName}{FileExtension} - end");
				}
			}
			catch (Exception ex)
			{
				staticAppData = new StaticAppData();
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, $"Exception raised loading staticAppData: {ex} - GetStaticData()");
			}
			finally
			{
				output = staticAppData;
				if(output==null)
				{
					await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, "Static data not loaded - GetStaticData()");
				}
				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "INIT DATA - SemaphoreStaticData.Release");
				semaphoreStaticData.Release();
			}

			return output;
		}
		internal static async Task<MessageAppData> GetMessageData()
		{
			MessageAppData output = null;
			try
			{
				if (messageAppData == null)
				{
					await semaphoreMessageData.WaitAsync();
					messageAppData = await LoadMessageData();
					await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE,
						$"Loading Message data from path - {FileInfo.MessageDataFileName}{FileExtension} - end");
				}
			}
			catch (Exception ex)
			{
				messageAppData = new MessageAppData();
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, $"Exception raised loading messageAppData {ex} - GetMessageData()");
			}
			finally
			{
				output = messageAppData; 
				if (output == null)
				{
					await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, "Message data not loaded - GetMessageData()");
				}
				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "INIT DATA - SemaphoreMessageData.Release");
				semaphoreMessageData.Release();
			}

			return output;
		}
		internal static async Task<ProgramAppData> GetProgramData()
		{
			ProgramAppData output = null;
			try
			{
				if (programAppData == null)
				{
					await semaphoreProgramData.WaitAsync();
					programAppData = await LoadProgramData();
					await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, $"Loading Program data from path - {FileInfo.ProgramDataFileName}{FileExtension} - end");
				}
			}
			catch (Exception ex)
			{
				programAppData = new ProgramAppData();
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, $"Exception raised loading programAppData {ex} - GetProgramData()");
			}
			finally
			{
				output = programAppData;
				if (output == null)
				{
					await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, "Program data not loaded - GetProgramData()");
				}
				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "INIT DATA - SemaphoreProgramData.Release");
				semaphoreProgramData.Release();
			}
			return output;
		}
		internal static async Task<ResultAppData> GetResultData()
		{
			ResultAppData output = null;
			try
			{
				if (resultAppData == null)
				{
					await semaphoreResultsData.WaitAsync();
					resultAppData = await LoadResultData();
					await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, $"Loading Result data from path - {FileInfo.ResultDataFileName}{FileExtension} - end");
				}
			}
			catch (Exception ex)
			{
				resultAppData = new ResultAppData();
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, $"Exception raised loading resultAppData {ex} - GetResultData()");
			}
			finally
			{
				output = resultAppData;
				if (output == null)
				{
					await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, "Result data not loaded - GetResultData()");
				}
				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "INIT DATA - SemaphoreResultData.Release");
				semaphoreResultsData.Release();
			}
			return output;
		}


		internal static async Task<int> UpdateStaticData(byte[] data){
			StaticAppData remote = null;
			try
			{
				if (data != null)
				{
					string response;
					using (var stream = new MemoryStream(data))
					using (var streamreader = new StreamReader(stream))
					{
						response = await streamreader.ReadToEndAsync();
					}

					var serializer = new JsonSerializer();
					using (TextReader reader = new StringReader(response))
					{
						var jsonreader = new JsonTextReader(reader);
						remote = serializer.Deserialize<StaticAppData>(jsonreader);
					}
				}
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR,
					$"Service returned unexpected response, data may be corrupt ex: {ex}, DataManager.UpdateStaticData");
			}

			if(remote == null || remote?.Version == 0){
				return 0;
			}

			StaticAppData local = await GetStaticData();

			local = local.Merge(remote);
			staticAppData = local;

			string json = JsonConvert.SerializeObject(local);

			byte[] memory = Encoding.UTF8.GetBytes(json);

			await SaveNewStaticData(memory);

			return remote.Version;
		}
		internal static async Task<int> UpdateMessageData(byte[] data)
		{
			MessageAppData remote = null;
			try
			{
				if (data != null)
				{
					string response;
					using (var stream = new MemoryStream(data))
					using (var streamreader = new StreamReader(stream))
					{
						response = await streamreader.ReadToEndAsync();
					}

					var serializer = new JsonSerializer();
					using (TextReader reader = new StringReader(response))
					{
						var jsonreader = new JsonTextReader(reader);
						remote = serializer.Deserialize<MessageAppData>(jsonreader);
					}
				}
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR,
					$"Service returned unexpected response, data may be corrupt ex: {ex}, DataManager.UpdateMessageData");
			}

			if (remote == null || remote?.Version == 0)
			{
				return 0;
			}

			MessageAppData local = await GetMessageData();

			local = local.Merge(remote);
			messageAppData = local;

			string json = JsonConvert.SerializeObject(local);

			byte[] memory = Encoding.UTF8.GetBytes(json);

			await SaveNewMessageData(memory);

			return remote.Version;
		}
		internal static async Task<int> UpdateProgramData(byte[] data)
		{
			ProgramAppData remote = null;
			try
			{
				if (data != null)
				{
					string response;
					using (var stream = new MemoryStream(data))
					using (var streamreader = new StreamReader(stream))
					{
						response = await streamreader.ReadToEndAsync();
					}

					var serializer = new JsonSerializer();
					using (TextReader reader = new StringReader(response))
					{
						var jsonreader = new JsonTextReader(reader);
						remote = serializer.Deserialize<ProgramAppData>(jsonreader);
					}
				}
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR,
					$"Service returned unexpected response, data may be corrupt ex: {ex}, DataManager.UpdateProgramData");
			}

			if (remote == null || remote?.Version == 0)
			{
				return 0;
			}

			ProgramAppData local = await GetProgramData();

			local = local.Merge(remote);
			programAppData = local;

			string json = JsonConvert.SerializeObject(local);

			byte[] memory = Encoding.UTF8.GetBytes(json);

			await SaveNewProgramData(memory);

			return remote.Version;
		}
		internal static async Task<int> UpdateResultsData(byte[] data)
		{
			ResultAppData remote = null;
			try
			{
				if (data != null)
				{
					string response;
					using (var stream = new MemoryStream(data))
					using (var streamreader = new StreamReader(stream))
					{
						response = await streamreader.ReadToEndAsync();
					}

					var serializer = new JsonSerializer();
					using (TextReader reader = new StringReader(response))
					{
						var jsonreader = new JsonTextReader(reader);
						remote = serializer.Deserialize<ResultAppData>(jsonreader);
					}
				}
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR,
					$"Service returned unexpected response, data may be corrupt ex: {ex}, DataManager.UpdateResultData");
			}

			if (remote == null || remote?.Version == 0)
			{
				return 0;
			}

			ResultAppData local = await GetResultData();

			local = local.Merge(remote);
			resultAppData = local;

			string json = JsonConvert.SerializeObject(local);

			byte[] memory = Encoding.UTF8.GetBytes(json);

			await SaveNewResultsData(memory);

			return remote.Version;
		}

		public static async Task<int> ClearStaticData(bool allow = false)
		{
			StaticAppData data = new StaticAppData();
			if (allow)
			{
				string json = JsonConvert.SerializeObject(data);

				staticAppData = data;
				byte[] memory = Encoding.UTF8.GetBytes(json);

				await SaveNewStaticData(memory);
				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "STATIC DATA CLEARED");
				SettingsManager.Instance.StaticVersion = data.Version;
			}
			return data.Version;
		}
		public static async Task<int> ClearMessageData(bool allow = false)
		{
			MessageAppData data = new MessageAppData();
			if (allow)
			{
				string json = JsonConvert.SerializeObject(data);

				messageAppData = data;
				byte[] memory = Encoding.UTF8.GetBytes(json);

				await SaveNewMessageData(memory);
				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "MESSAGE DATA CLEARED");
				SettingsManager.Instance.MessageVersion = data.Version;
			}
			return data.Version;
		}
		public static async Task<int> ClearProgramData(bool allow = false)
		{
			ProgramAppData data = new ProgramAppData();
			if (allow)
			{
				string json = JsonConvert.SerializeObject(data);

				programAppData = data;
				byte[] memory = Encoding.UTF8.GetBytes(json);

				await SaveNewProgramData(memory);
				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "PROGRAM DATA CLEARED");
				SettingsManager.Instance.ProgramVersion = data.Version;
			}
			return data.Version;
		}
		public static async Task<int> ClearResultData(bool allow = false)
		{
			ResultAppData data = new ResultAppData();
			if (allow)
			{
				string json = JsonConvert.SerializeObject(data);

				resultAppData = data;
				byte[] memory = Encoding.UTF8.GetBytes(json);

				await SaveNewResultsData(memory);
				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "RESULT DATA CLEARED");
				SettingsManager.Instance.ResultsVersion = data.Version;
			}
			return data.Version;
		}

		#region Save metody

		/// <summary>
		/// Uloží stažený XML soubor v byte[] do interního uložiště
		/// </summary>
		private static async Task SaveNewStaticData(byte[] data)
		{
			try
			{
				await semaphoreStaticData.WaitAsync();
				await InternalStorage.CopyTo(data, FileInfo.StaticDataFileName + FileExtension);
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, ex.Message, true);
			}
			finally
			{
				semaphoreStaticData.Release();
			}
		}

		/// <summary>
		/// Uloží stažený XML soubor v byte[] do interního uložiště
		/// </summary>
		private static async Task SaveNewMessageData(byte[] data)
		{
			try
			{
				await semaphoreMessageData.WaitAsync();
				await InternalStorage.CopyTo(data, FileInfo.MessageDataFileName + FileExtension);
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, ex.Message, true);
			}
			finally
			{
				semaphoreMessageData.Release();
			}
		}

		/// <summary>
		/// Uloží stažený XML soubor v byte[] do interního uložiště
		/// </summary>
		private static async Task SaveNewProgramData(byte[] data)
		{
			try
			{
				await semaphoreProgramData.WaitAsync();
				await InternalStorage.CopyTo(data, FileInfo.ProgramDataFileName + FileExtension);
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, ex.Message, true);
			}
			finally
			{
				semaphoreProgramData.Release();
			}
		}

		/// <summary>
		/// Uloží stažený XML soubor v byte[] do interního uložiště
		/// </summary>
		private static async Task SaveNewResultsData(byte[] data)
		{
			try
			{
				await semaphoreResultsData.WaitAsync();
				await InternalStorage.CopyTo(data, FileInfo.ResultDataFileName + FileExtension);
			}
			catch (Exception ex)
			{
				await ErrorLog.Instance.AddRecord(ErrorTypes.ERROR, ex.Message, true);
			}
			finally
			{
				semaphoreResultsData.Release();
			}
		}

		#endregion

		#region Load metody
		/// <summary>
		/// čte soubor v interním uložišti aplikace a vrátí jeho deserializovaný obsah
		/// </summary>
		/// <returns>deserializovaný obsah souboru</returns>
		private static async Task<StaticAppData> LoadStaticData()
		{
			await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, $"Loading static data from path - {FileInfo.StaticDataFileName+FileExtension}");
			var fileManager = Device.Instance.GetInternalFileManager(FileInfo.StaticDataFileName+FileExtension);
			string text = await fileManager.ReadAllText();

			using (TextReader reader = new StringReader(text))
			{
				var serializer = new JsonSerializer();
				var jsonreader = new JsonTextReader(reader);
				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "Deserialize - DataManager.LoadStaticData - Begin");

				var output = (StaticAppData)serializer.Deserialize<StaticAppData>(jsonreader);

				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "Deserialize - DataManager.LoadStaticData - End");
				return output;
			}
		}

		/// <summary>
		/// čte soubor v interním uložišti aplikace a vrátí jeho deserializovaný obsah
		/// </summary>
		/// <returns>deserializovaný obsah souboru</returns>
		private static async Task<MessageAppData> LoadMessageData()
		{
			await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, $"Loading program data from path - {FileInfo.MessageDataFileName+FileExtension}");
			var fileManager = Device.Instance.GetInternalFileManager(FileInfo.MessageDataFileName+FileExtension);
			string text = await fileManager.ReadAllText();
			using (TextReader reader = new StringReader(text))
			{
				var serializer = new JsonSerializer();
				var jsonreader = new JsonTextReader(reader);

				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "Deserialize - DataManager.LoadMessageData - Begin");

				var output = (MessageAppData)serializer.Deserialize<MessageAppData>(jsonreader);

				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "Deserialize - DataManager.LoadMessageData - End");
				return output;
			}
		}

		/// <summary>
		/// čte soubor v interním uložišti aplikace a vrátí jeho deserializovaný obsah
		/// </summary>
		/// <returns>deserializovaný obsah souboru</returns>
		private static async Task<ProgramAppData> LoadProgramData()
		{
			await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, $"Loading Program data from path - {FileInfo.ProgramDataFileName+FileExtension}");
			var fileManager = Device.Instance.GetInternalFileManager(FileInfo.ProgramDataFileName+FileExtension);
			string text = await fileManager.ReadAllText();
			using (TextReader reader = new StringReader(text))
			{
				var serializer = new JsonSerializer();
				var jsonreader = new JsonTextReader(reader);

				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "Deserialize - DataManager.LoadProgramData - Begin");

				var output = (ProgramAppData)serializer.Deserialize<ProgramAppData>(jsonreader);

				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "Deserialize - DataManager.LoadProgramData - End");
				return output;
			}
		}

		/// <summary>
		/// čte soubor v interním uložišti aplikace a vrátí jeho deserializovaný obsah
		/// </summary>
		/// <returns>deserializovaný obsah souboru</returns>
		private static async Task<ResultAppData> LoadResultData()
		{
			await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, $"Loading Result data from path - {FileInfo.ResultDataFileName+FileExtension}");
			var fileManager = Device.Instance.GetInternalFileManager(FileInfo.ResultDataFileName+FileExtension);
			string text = await fileManager.ReadAllText();
			using (TextReader reader = new StringReader(text))
			{
				var serializer = new JsonSerializer();
				var jsonreader = new JsonTextReader(reader);

				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "Deserialize - DataManager.LoadResultData - Begin");

				var output = (ResultAppData)serializer.Deserialize<ResultAppData>(jsonreader);

				await ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE, "Deserialize - DataManager.LoadResultData - End");
				return output;
			}
		}

		private static T DeserializeMemory<T>(byte[] param)
		{
			using (MemoryStream ms = new MemoryStream(param))
			{
				IFormatter br = new BinaryFormatter();
				return (T)br.Deserialize(ms);
			}
		}
		#endregion
	}
}