﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Input;
using ODM.Backend.Models.Enums;
using ODM.Backend.Models.Models;
using Skeleton.Mobile.Framework.Controls.Forms.Tools;
using Skeleton.Mobile.Framework.Controls.Forms.ViewModels;
using Skeleton.Mobile.Framework.Device;
using Skeleton.Mobile.Framework.Device.Location;
using Skeleton.Mobile.Framework.Error.Log;
using Skeleton.Mobile.Framework.Tools.Collections;
using Skeleton.Mobile.ODM.Converters;
using Skeleton.Mobile.ODM.Models.Events;
using Skeleton.Mobile.ODM.Tools;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;
using Skeleton.Mobile.ODM.Views;
using Gps = Skeleton.Mobile.Framework.Device.Location.Gps;

namespace Skeleton.Mobile.ODM.ViewModels
{
	public class VenueListViewModel : CancellationViewModel
	{
		private ObservableCollection<VenueItemViewModel> venues;
		public List<Venue> VenueModels { get; set; }

		public Action<bool> IconChange;

		public VenueTypeEnum CurrentTypeFilter
		{
			get { return currentTypeFilter; }
			set
			{
				currentTypeFilter = value; 
				OnPropertyChanged();
			}
		}

		public string CurrentCityFilter
		{
			get { return this.currentCityFilter;}
			set
			{
				currentCityFilter = value;
				OnPropertyChanged();
			}
		}

		private bool isFilterActive;
		public bool IsFilterActive
		{
			get { return this.isFilterActive; }
			set
			{
				this.isFilterActive = value;
				IconChange?.Invoke(value);
				this.OnPropertyChanged();
			}
		}

		private bool isTypeFilterActive;

		public bool IsTypeFilterActive
		{
			get { return isTypeFilterActive; }
			set { isTypeFilterActive = value; OnPropertyChanged(); }
		}

		private bool isCityFilterActive;

		public bool IsCityFilterActive
		{
			get { return isCityFilterActive; }
			set { isCityFilterActive = value; OnPropertyChanged();}
		}



		private string venueListPageTitle;
		public string VenueListPageTitle
		{
			get { return this.venueListPageTitle; }
			set
			{
				this.venueListPageTitle = value;
				this.OnPropertyChanged();
			}
		}

		private CancellationTokenSource locationCancelSource;
		private Gps currentLocation;
		private VenueTypeEnum currentTypeFilter;
		private string currentCityFilter;
		private const int LocationTimeout = 10000;

		public VenueListViewModel(List<Venue> venueList)
		{
			this.FilterClickedCommand = new SafetyCommand(this.FilterClickedCommand_Execute);
			this.VenueListPageTitle = "VenueList_Title".LocalizeResx();
			this.IsFilterActive = false;

			venues = new ObservableCollection<VenueItemViewModel>();
			if(venueList != null)
			{
				foreach (var venue in venueList)
				{
					venues.Add(new VenueItemViewModel(venue));
				}
			}
		}

		#region Properties

		public ObservableCollection<VenueItemViewModel> Venues
		{
			get
			{
				if (!isFilterActive)
				{
					return venues;
				}
				else
				{
					if (IsTypeFilterActive)
					{
						return new ObservableCollection<VenueItemViewModel>(venues.Where(v => v.Type == CurrentTypeFilter).ToList());
					}

					if (IsCityFilterActive)
					{
						return new ObservableCollection<VenueItemViewModel>(venues.Where(v => v.City == CurrentCityFilter).ToList());
					}

					return venues;
				}
			}
		}

		public VenueItemViewModel SelectedItem
		{
			get => null;
			set
			{
				if (value != null)
				{
					value.OpenDetail();
					OnPropertyChanged();
				}
			}
		}
		#endregion

		#region Location

		private async Task StartSearchingLocation()
		{
			this.currentLocation = null;
			this.locationCancelSource = new CancellationTokenSource();

			await
				ErrorLog.Instance.AddRecord(ErrorTypes.MESSAGE,
					$"GPS enabled - {Framework.Device.Device.Instance.LocationManager.GpsLocationEnabled}, Network enabled - {Framework.Device.Device.Instance.LocationManager.NetworkLocationEnabled}",
					"VenueListViewModel - searching location", false);

#pragma warning disable 4014
			Device.Instance.LocationManager.GetCurrentNetworkLocation(this.locationCancelSource.Token, LocationTimeout).ContinueWith(OnNetworkLocationFound);

			if (this.locationCancelSource != null)
			{
				Device.Instance.LocationManager.GetCurrentGpsLocation(this.locationCancelSource.Token, LocationTimeout).ContinueWith(OnGpsLocationFound);
			}

			if (this.locationCancelSource != null)
			{
				Device.Instance.LocationManager.GetFusedLocation(this.locationCancelSource.Token, LocationTimeout).ContinueWith(OnGoogleApiFusedPositionFound);
			}
#pragma warning restore 4014
		}

		private void StopSearchingLocation()
		{
			if (this.locationCancelSource != null)
			{
				this.locationCancelSource.Cancel();
				this.locationCancelSource = null;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="location">null if position services not available/allowed</param>
		private void UpdateLocation(Gps location)
		{
			if (location != null)
			{
				this.currentLocation = location;
				StopSearchingLocation();
				foreach (var venue in Venues)
				{
					venue.CurrentUserLocation = currentLocation;
					venue.ShowDistance = true;
				}
			}
			else
			{
				foreach (var venue in Venues)
				{
					venue.ShowDistance = false;
				}
			}
		}

		private void OnNetworkLocationFound(Task<Gps> task)
		{
			if (this.currentLocation == null || this.currentLocation.Provider != LocationProvider.Gps)
			{
				if (task.Result != null && task.Exception == null)
				{
					this.UpdateLocation(task.Result);
				}
			}
		}

		private void OnGpsLocationFound(Task<Gps> task)
		{
			if (this.currentLocation == null || this.currentLocation.Provider != LocationProvider.Gps ||
				this.currentLocation.Provider != LocationProvider.Network)
			{
				if (task.Result != null && task.Exception == null)
				{
					this.UpdateLocation(task.Result);
				}
			}
		}

		private void OnGoogleApiFusedPositionFound(Task<Gps> task)
		{
			if (task.Result != null && task.Exception == null)
			{
				this.UpdateLocation(task.Result);
			}
		}
		#endregion

		protected override async Task<bool> LoadDataInternal()
		{
			await base.LoadDataInternal();
			if (await Device.Instance.PermissionsManager.RequestLocationInUsePermission())
			{
				await StartSearchingLocation();
			}
			else
			{
				UpdateLocation(null);
			}

			return true;
		}

		#region Events

		private void Filter_VenueTypeChanged(object sender, VenueTypeChangedEventArgs args)
		{
			this.IsFilterActive = true;
			this.isTypeFilterActive = true;
			this.CurrentTypeFilter = args.SelectedType;
			var conv = new VenueTypeToNameConverter();
			this.VenueListPageTitle = (string)conv.Convert(args.SelectedType, null, true, null);
			OnPropertyChanged(nameof(VenueListPageTitle));
			OnPropertyChanged(nameof(Venues));
		}

		private void Filter_VenueCityChanged(object sender, VenueCityChangedEventArgs args)
		{
			this.IsFilterActive = true;
			this.isCityFilterActive = true;
			this.CurrentCityFilter = args.SelectedCity;
			this.VenueListPageTitle = args.SelectedCity;
			OnPropertyChanged(nameof(VenueListPageTitle));
			OnPropertyChanged(nameof(Venues));
		}

		#endregion


		#region Commands


		public ICommand FilterClickedCommand { get; private set; }

		private async Task FilterClickedCommand_Execute()
		{
			if (IsFilterActive)
			{
				this.DeactivateFilter();
			}
			else
			{
				var filterPage = new VenueListFilterRootPage();
				filterPage.VenueTypeChanged += Filter_VenueTypeChanged;
				filterPage.VenueCityChanged += Filter_VenueCityChanged;

				await RootPage.RootNavigationPage.PushAsync(filterPage, true);
			}
		}

		private void DeactivateFilter()
		{
			this.IsFilterActive = false;
			this.IsTypeFilterActive = false;
			this.VenueListPageTitle = "VenueList_Title".LocalizeResx();
			OnPropertyChanged(nameof(Venues));
		}

		#endregion
	}

}