﻿using System.Threading.Tasks;
using Skeleton.Mobile.Framework.Controls.Forms.Inputs;
using Skeleton.Mobile.Framework.Controls.Forms.Pages;
using Skeleton.Mobile.ODM.ViewModels.Abstract;
using Skeleton.Mobile.ODM.ViewModels.ItemViewModels;
using Skeleton.Mobile.ODM.Views.CustomViews;
using Xamarin.Forms;

namespace Skeleton.Mobile.ODM.Views.Abstract
{
	public abstract class TilesPage : ODMPage
	{
		public int MinDimension => 110;
		public int Margin
		{
			get
			{
				if (Xamarin.Forms.Device.Idiom == TargetIdiom.Tablet) return 2;
				else return 1;
			}
		}

		private ScrollView rootScrollView;
		private ContentView hidingView;
		private Grid tilesGrid;

		protected TilesViewModel viewModel;

		public void Initialize(ScrollView rootScrollView, ContentView hidingView, Grid tilesGrid)
		{
			this.rootScrollView = rootScrollView;
			this.hidingView = hidingView;
			this.tilesGrid = tilesGrid;
		}

		protected override async void OnAppearing()
		{
			base.OnAppearing();
			if (this.rootScrollView.ScrollY > 0) // pokud je nascrollováno, tak nascrolluje na danou pozici
			{
				await Task.Yield();
				await this.rootScrollView.ScrollToAsync(0, this.rootScrollView.ScrollY, false);
			}
		}

		protected override void OnViewLoad(double width, double height, OrientationMode orientation)
		{
			base.OnViewLoad(width, height, orientation);
			this.GenerateView(width, height, orientation);
		}

		protected void BaseOnViewLoad(double width, double height, OrientationMode orientation)
		{
			base.OnViewLoad(width, height, orientation);
		}

		protected override void OnOrientationChanged(double width, double height, OrientationMode orientation)
		{
			base.OnOrientationChanged(width, height, orientation);
			this.GenerateView(width, height, orientation);
		}

		protected void GenerateView(double width, double height, OrientationMode orientation)
		{
			var spacing = this.Margin * 2;
			this.tilesGrid.Margin = new Thickness(spacing, spacing * 5, spacing, spacing);
			if(this.viewModel != null && this.viewModel.TileItems !=null)
			this.GenerateTiles(width, orientation == OrientationMode.Portraint);

		}

		/// <summary>
		/// vygeneruje dlaždice
		/// </summary>
		/// <param name="width">šířka displeje</param>
		/// <param name="portrait">orientace</param>
		private void GenerateTiles(double width, bool portrait)
		{
			// výpočet rozměrů
			int columnCount = (int)width / this.MinDimension; // vypočítá počet sloupečků
			if (portrait && columnCount > 4) columnCount = 4; // nastaví maximální počet při orientaci na výšku
			else if (!portrait && columnCount > 6) columnCount = 6; // nastaví maximální počet při orientaci na šířku
			var cellDimension = (width / columnCount) - (2 * this.Margin); // vypočítá rozměry dlaždice
			int rowCount = viewModel.TileItems.Count / columnCount; // vypočítá počet řádků
			if (viewModel.TileItems.Count % columnCount > 0) rowCount++; // pokud není počet prvků dělitelný počtem sloupečků, tak se ještě musí přičíst řádek pro přebývající dlaždice

			this.tilesGrid.ColumnDefinitions.Clear();
			this.tilesGrid.RowDefinitions.Clear();
			this.tilesGrid.Children.Clear();

			// vytvoření mřížky
			for (int c = 0; c < columnCount; c++)
			{
				this.tilesGrid.ColumnDefinitions.Add(new ColumnDefinition { Width = new GridLength(1, GridUnitType.Star) });
			}
			for (int r = 0; r < rowCount; r++)
			{
				this.tilesGrid.RowDefinitions.Add(new RowDefinition { Height = new GridLength(1, GridUnitType.Auto) });
			}

			var index = 0;

			// naplnení mřížky
			for (int i = 0; i < rowCount; i++)
			{
				for (int j = 0; j < columnCount; j++)
				{
					var tile = this.CreateTile(viewModel.TileItems[index], cellDimension, viewModel.TileItems[index].Color != null ? Color.FromRgba(viewModel.TileItems[index].Color.Value.R, viewModel.TileItems[index].Color.Value.G, viewModel.TileItems[index].Color.Value.B, viewModel.TileItems[index].Color.Value.A) : Color.Gray);

					this.tilesGrid.Children.Add(tile);
					Grid.SetRow(tile, i);
					Grid.SetColumn(tile, j);
					index++;
					if (index >= viewModel.TileItems.Count) return; // Po překročení rozsahu pole je generování ukončeno
				}
			}
		}

		/// <summary>
		/// vytvoří view dlaždice
		/// </summary>
		/// <param name="item"></param>
		/// <param name="cellDimension">rozměry dlaždice</param>
		/// <param name="color">barva dlaždice</param>
		/// <returns>view dlaždice</returns>
		private ContentView CreateTile(TileItemViewModel item, double cellDimension, Color color)
		{
			var tile = new ContentButton
			{
				HeightRequest = cellDimension,
				WidthRequest = cellDimension,
				BackgroundColor = color,
				Margin = new Thickness(this.Margin),
				Command = item.tileCommand
			};

			var panel = new StackLayout
			{
				Spacing = 0,
				VerticalOptions = new LayoutOptions(LayoutAlignment.Center, false),
				HorizontalOptions = new LayoutOptions(LayoutAlignment.Center, false)
			};

			if (item.IconImage)
			{
				var icon = new IconLabel
				{
					FontFamily = item.FontFamily,
					FontName = "",
					Text = item.Icon,
					FontSize = (double)App.Current.Resources["TileIcon"],
					HorizontalOptions = new LayoutOptions(LayoutAlignment.Center, false),
					TextColor = Color.White
				};
				panel.Children.Add(icon);
			}
			else
			{
				var icon = new Image()
				{
					Source = item.Source,

				};
				panel.Children.Add(icon);
			}

			var label = new Label
			{
				Text = item.Title,
				Style = App.Current.Resources["TileLabel"] as Style,
				TextColor = Color.White
			};
			panel.Children.Add(label);

			tile.Content = panel;
			return tile;
		}
	}
}