﻿using System.Linq;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using ODMWeb.DL.Entities.Serialization;

namespace ODMWeb.DL.Repositories
{
	public class SerializationRecordRepository
	{
		protected readonly OdmDbContext dbContext;

		public SerializationRecordRepository(OdmDbContext dbContext)
		{
			this.dbContext = dbContext;
		}
		/// <summary>
		/// Returns all records of all types of files
		/// </summary>
		public IQueryable<SerializationRecordEntity> GetAll()
		{
			return dbContext.SerializedFiles.Where(f=>f.IsAvailable);
		}
		/// <summary>
		/// Gets specific record of given type by it's version
		/// </summary>
		public async Task<SerializationRecordEntity> GetByVersionAndType(int version, SerializationRecordType type)
		{
			var files = await dbContext.SerializedFiles.Where(r => r.Type == type && r.IsAvailable).ToListAsync();

			if (files.Count > 0 && version!=0)
			{
				return files.Find(f => f.Version == version);
			}
			else
			{
				return null;
			}
		}
		/// <summary>
		/// Gets last record of certain type
		/// </summary>
		public async Task<SerializationRecordEntity> GetLastAvailableByType(SerializationRecordType type)
		{
			var files = await dbContext.SerializedFiles.Where(r => r.Type == type && r.IsAvailable).ToListAsync();
			if (files.Count > 0)
			{
				return files.Last();
			}
			else
			{
				return null;
			}
		}

		/// <summary>
		/// Gets last record of certain type
		/// </summary>
		public async Task<SerializationRecordEntity> GetLastByType(SerializationRecordType type)
		{
			var files = await dbContext.SerializedFiles.Where(r => r.Type == type).ToListAsync();
			if (files.Count > 0)
			{
				return files.Last();
			}
			else
			{
				return null;
			}
		}

		public async Task<int> GetLastVersionByType(SerializationRecordType type)
		{
			var fileRecord = await GetLastByType(type);
			if (fileRecord != null)
			{
				return fileRecord.Version;
			}
			else
			{
				return 0;
			}
		}

		/// <summary>
		/// Adds new serialization record and autoincrements its version
		/// </summary>
		public async Task<SerializationRecordEntity> Add(SerializationRecordEntity record)
		{
			dbContext.Add(record);
			await dbContext.SaveChangesAsync();
			return record;
		}

		public async Task<SerializationRecordEntity> SetIsAvailable(SerializationRecordEntity entity, bool isAvailable)
		{
			var entityFromDb = await dbContext.SerializedFiles.FindAsync(entity.Id);
			entityFromDb.IsAvailable = isAvailable;
			await dbContext.SaveChangesAsync();
			return entityFromDb;
		}
	}
}