﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using ODM.Backend.Models.Models;
using ODMWeb.DL;
using ODMWeb.DL.Entities;
using ODMWeb.Models.ItemViewModels;
using ODMWeb.Models.SideProgramme;
using Serilog;

namespace ODMWeb.Controllers
{
	[Authorize]
	public abstract class AbsSideProgrammeController : Controller
	{
		protected IUnitOfWork unitOfWork;
		protected IMapper mapper;

		public Task<CompetitionEntity> GetCompTask;
		public ILogger logger;

		public AbsSideProgrammeController(IUnitOfWork unitOfWork, IMapper mapper)
		{
			this.unitOfWork = unitOfWork;
			this.mapper = mapper;
		}

		public async Task<IActionResult> Index()
		{
			var entity = await GetCompTask;
			var events = entity.Events.ToList();

			if (User.IsInRole("BenevolentDictatorForLife"))
			{
				events = events.Where(m => m.IsApproved || (!m.IsApproved && !m.IsRemoved)).ToList();
			}
			else
			{
				events = events.Where(m => (m.IsApproved && !m.IsRemoved) || (!m.IsApproved && !m.IsRemoved)).ToList();
			}

			var viewModel = SideProgrammeViewModel.Create(entity, events);

			return View(viewModel);
		}

		public async Task<IActionResult> CreateEvent()
		{
			CreateEventViewModel viewModel = new CreateEventViewModel();
			var competition = await GetCompTask;
			viewModel.CompetitionId = (competition).Id;
			viewModel.Competition = CompetitionItemViewModel.Create(competition);
			viewModel.Venues = mapper.Map<List<Venue>>(await unitOfWork.Venues.GetAll().ToListAsync());

			return View(viewModel);
		}


		[HttpPost]
		public async Task<IActionResult> CreateEvent(CreateEventViewModel viewModel)
		{
			if (!ModelState.IsValid)
			{
				return View(viewModel);
			}

			unitOfWork.Events.Add(CreateEventViewModel.CreateEntity(viewModel));
			await unitOfWork.Save();

			return RedirectToAction("Index");
		}
		
		public async Task<IActionResult> RemoveEvent(int id)
		{
			var eventEntity = await unitOfWork.Events.GetById(id);

			if (eventEntity.IsSerialized || eventEntity.IsApproved)
			{
				if (!User.IsInRole("BenevolentDictatorForLife"))
				{
					return RedirectToAction("Index");
				}
			}

			await unitOfWork.Events.Remove(eventEntity);
			eventEntity.IsModified = true;
			await unitOfWork.Save();

			var eventModel = mapper.Map<Event>(eventEntity);

			logger.Information("Event {@Event} removed by user {Name}.", eventModel, User.Identity.Name);

			return RedirectToAction("Index");
		}

		[Authorize(Roles = "BenevolentDictatorForLife")]
		public async Task<IActionResult> UnRemoveEvent(int id)
		{
			var eventEntity = await unitOfWork.Events.GetById(id);
			eventEntity.IsRemoved = false;
			await unitOfWork.Save();

			var eventModel = mapper.Map<Event>(eventEntity);
			logger.Information("Event {@Event} unremoved by user {Name}.", eventModel, User.Identity.Name);

			return RedirectToAction("Index");
		}

		public async Task<IActionResult> ModifyEvent(int id)
		{
			var eventEntity = await unitOfWork.Events.GetById(id);

			if (eventEntity.IsSerialized || eventEntity.IsApproved)
			{
				if (!User.IsInRole("BenevolentDictatorForLife"))
				{
					return RedirectToAction("Index");
				}
			}

			var eventModel = mapper.Map<Event>(eventEntity);

			var viewModel = ModifyEventViewModel.Create(eventModel);
			viewModel.Venues = mapper.Map<List<Venue>>(await unitOfWork.Venues.GetAll().ToListAsync());
			return View(viewModel);
		}

		[HttpPost]
		public async Task<IActionResult> ModifyEvent(ModifyEventViewModel viewModel)
		{
			if (!ModelState.IsValid)
			{
				viewModel.Venues = mapper.Map<List<Venue>>(await unitOfWork.Venues.GetAll().ToListAsync());
				return View(viewModel);
			}

			var eventEntity = await unitOfWork.Events.GetById(viewModel.Id);

			if (eventEntity.IsSerialized || eventEntity.IsApproved)
			{
				if (!User.IsInRole("BenevolentDictatorForLife"))
				{
					return RedirectToAction("Index");
				}
			}
			var eventModel = mapper.Map<Event>(eventEntity);
			logger.Information("Event {@Event} about to be modified by user {Name}.", eventModel, User.Identity.Name);

			eventEntity.Title = viewModel.Title;
			eventEntity.Start = viewModel.Start;
			eventEntity.Info = viewModel.Info;
			eventEntity.End = viewModel.End;
			eventEntity.IsModified = true;
			eventEntity.VenueId = viewModel.VenueId;
			unitOfWork.Events.Update(eventEntity);
			await unitOfWork.Save();
			eventModel = mapper.Map<Event>(eventEntity);
			logger.Information("Event {@Event} got modified by user {Name}.", eventModel, User.Identity.Name);

			return RedirectToAction("Index");
		}

		[Authorize(Roles = "BenevolentDictatorForLife,Overseer")]
		public async Task<IActionResult> ApproveEvent(int id)
		{
			var eventEntity = await unitOfWork.Events.GetById(id);
			if (eventEntity.IsSerialized)
			{
				return RedirectToAction("Index");
			}

			eventEntity.IsApproved = true;
			eventEntity.IsModified = false;
			await unitOfWork.Save();

			var eventModel = mapper.Map<Event>(eventEntity);
			logger.Information("Event {@Event} approved by user {Name}.", eventModel, User.Identity.Name);

			return RedirectToAction("Index");
		}

		[Authorize(Roles = "BenevolentDictatorForLife,Overseer")]
		public async Task<IActionResult> UnApproveEvent(int id)
		{
			var eventEntity = await unitOfWork.Events.GetById(id);
			if (eventEntity.IsSerialized)
			{
				return RedirectToAction("Index");
			}

			eventEntity.IsApproved = false;
			await unitOfWork.Save();

			var eventModel = mapper.Map<Event>(eventEntity);
			logger.Information("Event {@Event} unapproved by user {Name}.", eventModel, User.Identity.Name);

			return RedirectToAction("Index");
		}
	}
}