﻿using System;
using System.IO;
using System.Threading.Tasks;
using System.Xml.Serialization;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using ODM.Backend.Models.Models;
using ODM.Backend.Models.XmlSettings;
using ODMWeb.DataProviding.ApiProviders;
using ODMWeb.DL;
using ODMWeb.DL.Entities.Serialization;

namespace ODMWeb.Controllers.Api
{
	[Route("api/data")]
	[ApiController]
	public class OdmDataController : ControllerBase
	{
		private readonly IHostingEnvironment environment;
		private IUnitOfWork unitOfWork;

		public OdmDataController(ILogger<OdmDataController> logger, IHostingEnvironment environment, IUnitOfWork unitOfWork)
		{
			this.environment = environment;
			this.unitOfWork = unitOfWork;
		}

		[HttpGet("static/{remoteVersion}")]
		public async Task<StaticAppData> Static(int remoteVersion)
		{
			StaticAppData response = new StaticAppData();

			var type = SerializationRecordType.Static;

			var clientVersionRecord =
				await unitOfWork.SerializedFiles.GetByVersionAndType(remoteVersion, type);
			var lastServerVersionRecord =
				await unitOfWork.SerializedFiles.GetLastAvailableByType(type);

			if (lastServerVersionRecord?.Version == clientVersionRecord?.Version)
			{
				return response;
			}

			StaticApiProvider provider = new StaticApiProvider(this.unitOfWork);
			var clientAppData = await provider.GetData(clientVersionRecord) ?? new StaticAppData();
			var serverAppData = await provider.GetLastAvailableData(lastServerVersionRecord) ?? new StaticAppData();

			if (clientAppData?.Version == serverAppData?.Version)
			{
				return response;
			}

			response = provider.GetChanges(clientAppData, serverAppData);

			return response;
		}

		[HttpGet("message/{remoteVersion}")]
		public async Task<MessageAppData> Message(int remoteVersion)
		{
			MessageApiProvider provider = new MessageApiProvider(this.unitOfWork);
			MessageAppData response = new MessageAppData();

			var type = SerializationRecordType.Message;

			var clientVersionRecord =
				await unitOfWork.SerializedFiles.GetByVersionAndType(remoteVersion, type);
			var lastServerVersionRecord =
				await unitOfWork.SerializedFiles.GetLastAvailableByType(type);


			if (lastServerVersionRecord?.Version == clientVersionRecord?.Version)
			{
				return response;
			}

			var clientAppData = await provider.GetData(clientVersionRecord) ?? new MessageAppData();
			var serverAppData = await provider.GetLastAvailableData(lastServerVersionRecord) ?? new MessageAppData();

			if (clientAppData?.Version == serverAppData?.Version)
			{
				return response;
			}

			response = provider.GetChanges(clientAppData, serverAppData);

			return response;
		}

		[HttpGet("program/{remoteVersion}")]
		public async Task<ProgramAppData> Program(int remoteVersion)
		{
			ProgramApiProvider provider = new ProgramApiProvider(this.unitOfWork);
			ProgramAppData response = new ProgramAppData();

			var type = SerializationRecordType.Program;

			var clientVersionRecord =
				await unitOfWork.SerializedFiles.GetByVersionAndType(remoteVersion, type);
			var lastServerVersionRecord =
				await unitOfWork.SerializedFiles.GetLastAvailableByType(type);


			if (lastServerVersionRecord?.Version == clientVersionRecord?.Version)
			{
				return response;
			}

			var clientAppData = await provider.GetData(clientVersionRecord) ?? new ProgramAppData();
			var serverAppData = await provider.GetLastAvailableData(lastServerVersionRecord) ?? new ProgramAppData();

			if (clientAppData?.Version == serverAppData?.Version)
			{
				return response;
			}

			response = provider.GetChanges(clientAppData, serverAppData);

			return response;
		}

		[HttpGet("result/{remoteVersion}")]
		public async Task<ResultAppData> Result(int remoteVersion)
		{
			ResultApiProvider provider = new ResultApiProvider(this.unitOfWork);
			ResultAppData response = new ResultAppData();

			var type = SerializationRecordType.Result;

			var clientVersionRecord =
				await unitOfWork.SerializedFiles.GetByVersionAndType(remoteVersion, type);
			var lastServerVersionRecord =
				await unitOfWork.SerializedFiles.GetLastAvailableByType(type);


			if (lastServerVersionRecord?.Version == clientVersionRecord?.Version)
			{
				return response;
			}

			var clientAppData = await provider.GetData(clientVersionRecord) ?? new ResultAppData();
			var serverAppData = await provider.GetLastAvailableData(lastServerVersionRecord) ?? new ResultAppData();

			if (clientAppData?.Version == serverAppData?.Version)
			{
				return response;
			}

			response = provider.GetChanges(clientAppData, serverAppData);

			return response;
		}


		[HttpGet("staticexact/{version}")]
		[Authorize]
		public async Task<StaticAppData> StaticExact(int version)
		{
			StaticApiProvider provider = new StaticApiProvider(this.unitOfWork);

			var type = SerializationRecordType.Static;

			var clientVersionRecord =
				await unitOfWork.SerializedFiles.GetByVersionAndType(version, type);

			var data = await provider.GetData(clientVersionRecord) ?? new StaticAppData();

			return data;
		}

		[HttpGet("messageexact/{version}")]
		[Authorize]
		public async Task<MessageAppData> MessageExact(int version)
		{
			MessageApiProvider provider = new MessageApiProvider(this.unitOfWork);

			var type = SerializationRecordType.Message;

			var clientVersionRecord =
				await unitOfWork.SerializedFiles.GetByVersionAndType(version, type);

			var data = await provider.GetData(clientVersionRecord) ?? new MessageAppData();

			return data;
		}

		[HttpGet("programexact/{version}")]
		[Authorize]
		public async Task<ProgramAppData> ProgramExact(int version)
		{
			ProgramApiProvider provider = new ProgramApiProvider(this.unitOfWork);
			var type = SerializationRecordType.Program;

			var clientVersionRecord =
				await unitOfWork.SerializedFiles.GetByVersionAndType(version, type);

			var data = await provider.GetData(clientVersionRecord) ?? new ProgramAppData();

			return data;
		}

		[HttpGet("resultexact/{version}")]
		[Authorize]
		public async Task<ResultAppData> ResultExact(int version)
		{
			ResultApiProvider provider = new ResultApiProvider(this.unitOfWork);

			var type = SerializationRecordType.Result;

			var clientVersionRecord =
				await unitOfWork.SerializedFiles.GetByVersionAndType(version, type);

			var data = await provider.GetData(clientVersionRecord) ?? new ResultAppData();

			return data;
		}

		[HttpGet("state")]
		public async Task<FileState> GetFilesState()
		{
			FileState output = new FileState();

			SerializationRecordEntity lastStaticEntry =
				await unitOfWork.SerializedFiles.GetLastAvailableByType(SerializationRecordType.Static);
			SerializationRecordEntity lastProgramEntry =
				await unitOfWork.SerializedFiles.GetLastAvailableByType(SerializationRecordType.Program);
			SerializationRecordEntity lastResultEntry =
				await unitOfWork.SerializedFiles.GetLastAvailableByType(SerializationRecordType.Result);
			SerializationRecordEntity lastMessageEntry =
				await unitOfWork.SerializedFiles.GetLastAvailableByType(SerializationRecordType.Message);

			int lastStaticLocalVersion = 0, lastProgramFileVersion = 0, lastResultFileVersion = 0, lastMessageFileVersion = 0;

			if (lastStaticEntry != null) lastStaticLocalVersion = lastStaticEntry.Version;
			if (lastProgramEntry != null) lastProgramFileVersion = lastProgramEntry.Version;
			if (lastResultEntry != null) lastResultFileVersion = lastResultEntry.Version;
			if (lastMessageEntry != null) lastMessageFileVersion = lastMessageEntry.Version;

			output.LastStaticFileVersion = lastStaticLocalVersion;
			output.LastProgramFileVersion = lastProgramFileVersion;
			output.LastResultFileVersion = lastResultFileVersion;
			output.LastMessageFileVersion = lastMessageFileVersion;

			return output;
		}

	}
}