﻿using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using ODM.Backend.Models.Models;
using ODMWeb.DL;
using ODMWeb.DL.Entities;
using ODMWeb.Models.ItemViewModels;
using ODMWeb.Models.Menu;
using Serilog;

namespace ODMWeb.Controllers
{
	[Authorize]
	public class MenuController : Controller
	{
		private readonly ILogger logger = Log.ForContext<MenuController>();
		private readonly IUnitOfWork unitOfWork;
		private readonly IMapper mapper;
		public MenuController(IUnitOfWork unitOfWork, IMapper mapper)
		{
			this.unitOfWork = unitOfWork;
			this.mapper = mapper;
		}

		public async Task<IActionResult> Index()
		{
			var menus = await this.unitOfWork.Menus.GetAll().ToListAsync();
			var meals = await this.unitOfWork.Meals.GetAll().ToListAsync();


			if (User.IsInRole("BenevolentDictatorForLife"))
			{
				menus = menus.Where(m => m.IsApproved || (!m.IsApproved && !m.IsRemoved)).ToList();
				meals = meals.Where(m => (m.IsApproved && !m.IsRemoved) || (!m.IsApproved && !m.IsRemoved)).ToList();
			}
			else
			{
				menus = menus.Where(m => (m.IsApproved && !m.IsRemoved) || (!m.IsApproved && !m.IsRemoved)).ToList();
				meals = meals.Where(m => (m.IsApproved && !m.IsRemoved) || (!m.IsApproved && !m.IsRemoved)).ToList();
			}

			List<MenuItemViewModel> menuItemViewModels = mapper.Map<List<MenuItemViewModel>>(menus);
			List<MealItemViewModel> mealItemViewModels = mapper.Map<List<MealItemViewModel>>(meals);

			List<MenuViewModel> menuViewModels = MenuViewModel.CreateList(menuItemViewModels, mealItemViewModels);
			MenuListViewModel viewModel = new MenuListViewModel(menuViewModels);

			return View(viewModel);
		}

		public async Task<IActionResult> ModifyMenu(int id)
		{
			var entity = await unitOfWork.Menus.GetById(id);

			if (entity.IsSerialized || entity.IsApproved)
			{
				if (!User.IsInRole("BenevolentDictatorForLife"))
				{
					return RedirectToAction("Index");
				}
			}

			var menu = mapper.Map<MenuItemViewModel>(entity);
			var meals = mapper.Map<List<MealItemViewModel>>(entity.Meals);
			var viewModel = ModifyMenuViewModel.Create(MenuViewModel.CreateSingle(menu, meals), this.mapper);
			return View(viewModel);
		}

		[HttpPost]
		public async Task<IActionResult> ModifyMenu(ModifyMenuViewModel viewModel)
		{
			if (!ModelState.IsValid)
			{
				return View(viewModel);
			}

			var menuEntity = await unitOfWork.Menus.GetById(viewModel.Id);

			if (menuEntity.IsSerialized || menuEntity.IsApproved)
			{
				if (!User.IsInRole("BenevolentDictatorForLife"))
				{
					return RedirectToAction("Index");
				}
			}

			var menu = mapper.Map<Menu>(menuEntity);
			var meals = mapper.Map<List<Meal>>(menuEntity.Meals);

			logger.Information("Menu {@Menu} with meals {@Meals} about to be modified by user {Name}.", menu, meals, User.Identity.Name);

			bool gotModified = false;
			List<MealEntity> mealEntities = await unitOfWork.Meals.Find(m => m.MenuId == menuEntity.Id).ToListAsync();
			List<MealEntity> entitiesToDelete = new List<MealEntity>();

			foreach (var meal in mealEntities)
			{
				var updated = viewModel.Meals?.Find(m => m.Id == meal.Id);
				if (updated!=null)
				{
					var entity = await unitOfWork.Meals.GetById(updated.Id);
					entity.Date = updated.Date;
					entity.Text = updated.Text;
					entity.Type = updated.Type;
					entity.IsRemoved = updated.IsRemoved;
					viewModel.Meals.Remove(updated);
					unitOfWork.Meals.Update(entity);

					gotModified = true;
				}
				else
				{
					entitiesToDelete.Add(meal);
				}
			}

			if (viewModel.Meals != null)
				foreach (var meal in viewModel.Meals)
				{
					gotModified = true;
					menuEntity.Meals.Add(new MealEntity()
						{Date = meal.Date, MenuId = menuEntity.Id, Text = meal.Text, Type = meal.Type});
				}

			await unitOfWork.Meals.RemoveRange(entitiesToDelete);

			if (gotModified){
				menuEntity.IsModified = true;
			}
			await unitOfWork.Save();

			menu = mapper.Map<Menu>(menuEntity);
			meals = mapper.Map<List<Meal>>(menuEntity.Meals);

			logger.Information("Menu {@Menu} with meals {@Meals} modified by user {Name}.", menu, meals, User.Identity.Name);

			return RedirectToAction("Index");
		}

		public IActionResult CreateMenu()
		{
			CreateMenuViewModel viewModel = new CreateMenuViewModel();
			return View(viewModel);
		}

		[HttpPost]
		public async Task<IActionResult> CreateMenu(CreateMenuViewModel viewModel)
		{
			if (!ModelState.IsValid)
			{
				return View(viewModel);
			}
			unitOfWork.Menus.Add(viewModel.CreateEntity());
			await unitOfWork.Save();
			return RedirectToAction("Index");
		}

		public async Task<IActionResult> RemoveMenu(int id)
		{
			var menuEntity = await unitOfWork.Menus.GetById(id);

			if (menuEntity.IsSerialized || menuEntity.IsApproved)
			{
				if (!User.IsInRole("BenevolentDictatorForLife"))
				{
					return RedirectToAction("Index");
				}
			}

			await unitOfWork.Menus.Remove(menuEntity);
			await unitOfWork.Save();

			var menu = mapper.Map<Menu>(menuEntity);
			var meals = mapper.Map<List<Meal>>(menuEntity.Meals);

			logger.Information("Menu {@Menu} with meals {@Meals} removed by user {Name}.", menu, meals, User.Identity.Name);

			return RedirectToAction("Index");
		}

		[Authorize(Roles = "BenevolentDictatorForLife")]
		public async Task<IActionResult> UnRemoveMenu(int id)
		{
			var menuEntity = await unitOfWork.Menus.GetById(id);

			menuEntity.IsRemoved = false;

			await unitOfWork.Save();

			var menu = mapper.Map<Menu>(menuEntity);
			var meals = mapper.Map<List<Meal>>(menuEntity.Meals);

			logger.Information("Menu {@Menu} with meals {@Meals} unremoved by user {Name}.", menu, meals, User.Identity.Name);

			return RedirectToAction("Index");
		}

		[Authorize(Roles = "BenevolentDictatorForLife,Overseer")]
		public async Task<IActionResult> ApproveMenu(int id)
		{
			var menuEntity = await unitOfWork.Menus.GetById(id);
			if (menuEntity.IsSerialized)
			{
				return RedirectToAction("Index");
			}

			foreach (var mealEntity in menuEntity.Meals)
			{
				mealEntity.IsApproved = true;
			}

			menuEntity.IsApproved = true;
			menuEntity.IsModified = false;
			await unitOfWork.Save();

			var menu = mapper.Map<Menu>(menuEntity);
			var meals = mapper.Map<List<Meal>>(menuEntity.Meals);

			logger.Information("Menu {@Menu} with meals {@Meals} approved by user {Name}.", menu, meals, User.Identity.Name);

			return RedirectToAction("Index");
		}

		[Authorize(Roles = "BenevolentDictatorForLife,Overseer")]
		public async Task<IActionResult> UnApproveMenu(int id)
		{
			var menuEntity = await unitOfWork.Menus.GetById(id);
			if (menuEntity.IsSerialized)
			{
				return RedirectToAction("Index");
			}

			foreach (var mealEntity in menuEntity.Meals)
			{
				mealEntity.IsApproved = false;
			}

			menuEntity.IsApproved = false;
			await unitOfWork.Save();

			var menu = mapper.Map<Menu>(menuEntity);
			var meals = mapper.Map<List<Meal>>(menuEntity.Meals);

			logger.Information("Menu {@Menu} with meals {@Meals} unapproved by user {Name}.", menu, meals, User.Identity.Name);

			return RedirectToAction("Index");
		}

	}
}