﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using AutoMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using ODM.Backend.Models.Models;
using ODMWeb.DL;
using ODMWeb.DL.Entities;
using ODMWeb.Models.Message;
using Serilog;

namespace ODMWeb.Controllers
{
	[Authorize]
	public class MessageController : Controller
	{
		private readonly IUnitOfWork unitOfWork;
		private readonly IMapper mapper;
		private readonly ILogger logger = Log.ForContext<MessageController>();

		public MessageController(IUnitOfWork unitOfWork, IMapper mapper)
		{
			this.unitOfWork = unitOfWork;
			this.mapper = mapper;
		}

		public async Task<IActionResult> Index()
		{
			var messageEntities = await unitOfWork.Messages.GetAll().ToListAsync();

			if (User.IsInRole("BenevolentDictatorForLife"))
			{
				messageEntities = messageEntities.Where(m => m.IsApproved || (!m.IsApproved && !m.IsRemoved)).ToList();
			}
			else
			{
				messageEntities = messageEntities.Where(m => (m.IsApproved && !m.IsRemoved) || (!m.IsApproved && !m.IsRemoved)).ToList();
			}

			var viewModel = MessageListViewModel.Create(messageEntities, mapper);

			return View(viewModel);
		}

		public IActionResult CreateMessage()
		{
			var viewModel = new CreateMessageViewModel();
			return View(viewModel);
		}

		[HttpPost]
		public async Task<IActionResult> CreateMessage(CreateMessageViewModel viewModel)
		{
			if (!ModelState.IsValid)
			{
				return View(viewModel);
			}

			unitOfWork.Messages.Add(viewModel.CreateEntity());
			await unitOfWork.Save();
			return RedirectToAction("Index");
		}

		public async Task<IActionResult> ModifyMessage(int id)
		{
			var messageEntity = await unitOfWork.Messages.GetById(id);

			if (messageEntity.IsSerialized || messageEntity.IsApproved)
			{
				if (!User.IsInRole("BenevolentDictatorForLife"))
				{
					return RedirectToAction("Index");
				}
			}

			var message = mapper.Map<Message>(messageEntity);
			var viewModel = ModifyMessageViewModel.Create(message);
			return View(viewModel);
		}

		[HttpPost]
		public async Task<IActionResult> ModifyMessage(ModifyMessageViewModel viewModel)
		{
			if (!ModelState.IsValid)
			{
				return View(viewModel);
			}

			var messageEntity = await unitOfWork.Messages.GetById(viewModel.Id);

			var message = mapper.Map<Message>(messageEntity);

			logger.Information("Message {@Message} about to be modified by user {Name}.", message, User.Identity.Name);

			if (messageEntity.IsSerialized || messageEntity.IsApproved)
			{
				if (!User.IsInRole("BenevolentDictatorForLife"))
				{
					return RedirectToAction("Index");
				}
			}

			messageEntity.Title = viewModel.Title;
			messageEntity.Date = viewModel.Date;
			messageEntity.Text = viewModel.Text;
			messageEntity.Topic = viewModel.Topic;
			messageEntity.IsImportant = viewModel.IsImportant;
			messageEntity.IsModified = true;
			unitOfWork.Messages.Update(messageEntity);
			await unitOfWork.Save();

			message = mapper.Map<Message>(messageEntity);
			logger.Information("Message {@Message} modified by user {Name}.", message, User.Identity.Name);

			return RedirectToAction("Index");
		}

		public async Task<IActionResult> RemoveMessage(int id)
		{
			var messageEntity = await unitOfWork.Messages.GetById(id);

			if (messageEntity.IsSerialized || messageEntity.IsApproved)
			{
				if (!User.IsInRole("BenevolentDictatorForLife"))
				{
					return RedirectToAction("Index");
				}
			}
			await unitOfWork.Messages.Remove(messageEntity);
			await unitOfWork.Save();

			var message = mapper.Map<Message>(messageEntity);
			logger.Information("Message {@Message} removed by user {Name}.", message, User.Identity.Name);

			return RedirectToAction("Index");
		}

		[Authorize(Roles = "BenevolentDictatorForLife")]
		public async Task<IActionResult> UnRemoveMessage(int id)
		{
			var messageEntity = await unitOfWork.Messages.GetById(id);
			messageEntity.IsRemoved = false;
			await unitOfWork.Save();

			var message = mapper.Map<Message>(messageEntity);
			logger.Information("Message {@Message} unremoved by user {Name}.", message, User.Identity.Name);

			return RedirectToAction("Index");
		}

		[Authorize(Roles = "BenevolentDictatorForLife,Overseer")]
		public async Task<IActionResult> ApproveMessage(int id)
		{
			var messageEntity = await unitOfWork.Messages.GetById(id);
			if (messageEntity.IsSerialized)
			{
				return RedirectToAction("Index");
			}

			messageEntity.IsApproved = true;
			messageEntity.IsModified = false;
			await unitOfWork.Save();

			var message = mapper.Map<Message>(messageEntity);
			logger.Information("Message {@Message} approved by user {Name}.", message, User.Identity.Name);

			return RedirectToAction("Index");
		}

		[Authorize(Roles = "BenevolentDictatorForLife,Overseer")]
		public async Task<IActionResult> UnApproveMessage(int id)
		{
			var messageEntity = await unitOfWork.Messages.GetById(id);
			if (messageEntity.IsSerialized)
			{
				return RedirectToAction("Index");
			}

			messageEntity.IsApproved = false;
			await unitOfWork.Save();

			var message = mapper.Map<Message>(messageEntity);
			logger.Information("Message {@Message} unapproved by user {Name}.", message, User.Identity.Name);

			return RedirectToAction("Index");
		}
	}
}