﻿using System;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Xml.Serialization;
using ODM.Backend.Models.Models;
using ODMWeb.DL;
using ODMWeb.DL.Entities.Serialization;
using Serilog;

namespace ODMWeb.DataProviding.ApiProviders
{
	public class StaticApiProvider : IApiProvider<StaticAppData>
	{
		private readonly ILogger logger = Log.ForContext<StaticApiProvider>();
		private IUnitOfWork unitOfWork;
		public StaticApiProvider(IUnitOfWork unitOfWork)
		{
			this.unitOfWork = unitOfWork;
		}

		public StaticAppData GetChanges(StaticAppData client, StaticAppData server)
		{
			StaticAppData output = new StaticAppData() { Version = server.Version };

			output.Meals = server.Meals.Except(client.Meals, new MealComparer()).ToList();
			output.Menus = server.Menus.Except(client.Menus, new MenuComparer()).ToList();
			output.Cities = server.Cities;
			output.Regions = server.Regions.Except(client.Regions, new RegionComparer()).ToList();
			output.Sports = server.Sports.Except(client.Sports, new SportComparer()).ToList();
			output.Venues = server.Venues.Except(client.Venues, new VenueComparer()).ToList();

			return output;
		}

		public async Task<StaticAppData> GetData(SerializationRecordEntity record)
		{
			StaticAppData output = null;

			if (record == null || !File.Exists(record.FilePath))
			{
				return output;
			}

			try
			{
				using (var reader = new StreamReader(record.FilePath))
				{
					var serializer = new XmlSerializer(typeof(StaticAppData));

					output = (StaticAppData)serializer.Deserialize(reader.BaseStream);
				}
			}
			catch (Exception ex)
			{
				output = null;
				logger.Error(ex, "GetData encountered an exception.");
				await unitOfWork.SerializedFiles.SetIsAvailable(record, false);
				logger.Warning("Record no longer available: {@SerializationRecordEntity}", record);
			}

			return output;
		}

		public async Task<StaticAppData> GetLastAvailableData(SerializationRecordEntity record)
		{
			StaticAppData output = null;
			SerializationRecordEntity lastRecord = record;

			while (output == null && lastRecord != null)
			{
				output = await GetData(lastRecord);

				if (output == null)
				{
					lastRecord =
						await unitOfWork.SerializedFiles.GetLastAvailableByType(SerializationRecordType.Static);
				}
			}

			return output;
		}
	}
}