﻿using System;
using System.IO;
using System.Linq;
using System.Threading.Tasks;
using System.Xml.Serialization;
using AutoMapper;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using ODM.Backend.Models.Models;
using ODMWeb.Controllers.Api;
using ODMWeb.DL;
using ODMWeb.DL.Entities.Serialization.Groups;
using Serilog;
using ILogger = Serilog.ILogger;

namespace ODMWeb.DataProviding.Serialization
{
	public class StaticDataSerializer : IDataSerializer<StaticEntityData, StaticAppData>
	{
		private readonly ILogger logger = Log.ForContext<StaticDataSerializer>();

		public StaticAppData CreateDataGroup(IMapper mapper, StaticEntityData entities)
		{
			StaticAppData output;
			output = mapper.Map<StaticAppData>(entities);

			foreach (var outputSport in output.Sports)
			{
				outputSport.Item = new IdItem(outputSport.Item.Key, outputSport.Item.Name);
			}

			foreach (var outputVenue in output.Venues)
			{
				outputVenue.Address = new Address(outputVenue.Address.Street, outputVenue.Address.StreetNumber, outputVenue.Address.City);
				outputVenue.Location = new Gps(outputVenue.Location.Latitude, outputVenue.Location.Longitude);
			}

			return output;
		}

		public async Task<StaticEntityData> CreateEntityGroup(IUnitOfWork unitOfWork)
		{
			StaticEntityData output = new StaticEntityData
			{
				Menus = await unitOfWork.Menus.GetAll()
					.Where(e => e.IsApproved).ToListAsync(),
				Meals = await unitOfWork.Meals.GetAll()
					.Where(e => e.IsApproved).ToListAsync(),

				Regions = await unitOfWork.Regions.GetAll()
					.ToListAsync(),
				Sports = await unitOfWork.Sports.GetAll()
					.ToListAsync(),
				Venues = await unitOfWork.Venues.GetAll()
					.ToListAsync()
			};

			return output;
		}

		public bool SerializeData(StaticAppData data, string path, int version, IUnitOfWork unitOfWork = null)
		{
			var serializer = new XmlSerializer(typeof(StaticAppData));
			foreach (var venue in data.Venues)
			{
				if (!data.Cities.Contains(venue.Address.City))
				{
					data.Cities.Add(venue.Address.City);
				}
			}


			data.Version = version;

			using (var writer = new StreamWriter(path))
			{
				try
				{
					serializer.Serialize(writer, data);
				}
				catch (Exception ex)
				{
					logger.Error(ex, "Serializer encountered an exception.");
					return false;
				}
			}

			return true;
		}

		public async Task SetEntitiesPublished(StaticEntityData entities, IUnitOfWork unitOfWork)
		{
			foreach (var entity in entities.Meals)
			{
				entity.IsSerialized = true;
				entity.IsModified = false;
			}

			foreach (var entity in entities.Menus)
			{
				entity.IsSerialized = true;
				entity.IsModified = false;
			}

			foreach (var entity in entities.Regions)
			{
				entity.IsSerialized = true;
				entity.IsModified = false;
			}

			foreach (var entity in entities.Sports)
			{
				entity.IsModified = false;
				entity.IsSerialized = true;
			}

			foreach (var entity in entities.Venues)
			{
				entity.IsSerialized = true;
				entity.IsModified = false;
			}

			await unitOfWork.Save();
		}
	}
}