﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;
using AutoMapper;
using Hangfire;
using Hangfire.Logging;
using Hangfire.SqlServer;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Identity.UI;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.HttpsPolicy;
using Microsoft.AspNetCore.Localization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Razor;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using ODM.Backend.Models.Models;
using ODMWeb.Converters;
using ODMWeb.DataProviding;
using ODMWeb.DataProviding.Publishers;
using ODMWeb.DL;
using ODMWeb.DL.Entities;
using ODMWeb.DL.Entities.Serialization.Groups;
using ODMWeb.DL.Seed;
using ODMWeb.Hangfire;
using ODMWeb.Hangfire.RecurringJobs;
using ODMWeb.SharedLocalization;
using Serilog;

namespace ODMWeb
{
	public class Startup
	{
		private string ConnectionStringName => "DefaultConnection";

		public Startup(IConfiguration configuration)
		{
			this.Configuration = configuration;
		}

		private IConfiguration Configuration { get; }

		// This method gets called by the runtime. Use this method to add services to the container.
		public void ConfigureServices(IServiceCollection services)
		{
			#region dbContext

			services.AddDbContext<OdmDbContext>(options =>
				options
					.UseLazyLoadingProxies()
					.UseSqlServer(this.Configuration.GetConnectionString(this.ConnectionStringName))
			);

			#endregion

			#region Identity

			services.AddIdentity<IdentityUser, IdentityRole>().AddEntityFrameworkStores<OdmDbContext>()
				.AddDefaultTokenProviders();

			#endregion

			#region UnitOfWork

			services.AddScoped<IUnitOfWork, UnitOfWork>();

			#endregion

			#region Automapper

			var config = new MapperConfiguration(cfg =>
			{
				cfg.CreateMissingTypeMaps = true;
				cfg.Advanced.MaxExecutionPlanDepth = 1;
				cfg.CreateMap<CompetitionEntity, Competition>().ForMember(m => m.Cities, opt => opt.Ignore());
				cfg.CreateMap<MessageEntityData, MessageAppData>().ForMember(m => m.Version, opt => opt.Ignore());
				cfg.CreateMap<ProgramEntityData, ProgramAppData>().ForMember(m => m.Version, opt => opt.Ignore());
				cfg.CreateMap<ResultEntityData, ResultAppData>().ForMember(m => m.Version, opt => opt.Ignore());
				cfg.CreateMap<StaticEntityData, StaticAppData>().ForMember(m => m.Version, opt => opt.Ignore());
			});
			var mapper = config.CreateMapper();
			services.AddSingleton(mapper);

			#endregion

			#region Shared resources

			services.AddSingleton<LocService>();

			services.AddSingleton<MealTypeToTextConverter>();
			services.AddSingleton<MessageTopicToTextConverter>();
			services.AddSingleton<SerializedFileTypeToNameConverter>();

			services.AddLocalization(l => l.ResourcesPath = "Resources");
			services.AddMvc().AddViewLocalization().AddDataAnnotationsLocalization(options =>
				options.DataAnnotationLocalizerProvider = (type, factory) =>
				{
					var assemblyName = new AssemblyName(typeof(SharedResource).GetTypeInfo().Assembly.FullName);
					return factory.Create(nameof(Resources.Resources), assemblyName.Name);
				});

			services.Configure<RequestLocalizationOptions>(
				options =>
				{
					var supportedCultures = new List<CultureInfo>
					{
						new CultureInfo("cs-CZ")
					};
					options.DefaultRequestCulture = new RequestCulture(culture: "cs-CZ", uiCulture: "cs-CZ");
					options.SupportedCultures = supportedCultures;
					options.SupportedUICultures = supportedCultures;
					options.RequestCultureProviders.Insert(0, new QueryStringRequestCultureProvider());
				});

			#endregion

			services.AddHangfire(x => x.UseSqlServerStorage(this.Configuration.GetConnectionString(this.ConnectionStringName)));

			services.AddSingleton<IHttpContextAccessor, HttpContextAccessor>();
		}

		// This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
		public void Configure(IApplicationBuilder app, IHostingEnvironment env, IServiceProvider serviceProvider,
			UserManager<IdentityUser> userManager, RoleManager<IdentityRole> roleManager)
		{
			if (env.IsDevelopment())
			{
				app.UseDeveloperExceptionPage();
				app.UseDatabaseErrorPage();
			}
			else
			{
				app.UseExceptionHandler("/Home/Error");
				// The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
				app.UseHsts();
			}

			DatabaseInitializer.Seed(userManager, roleManager, serviceProvider.GetRequiredService<OdmDbContext>());

			#region Data folder init

			var dataFolderPath = Path.Combine(env.ContentRootPath.ToString(), Config.FolderName);
			if (!System.IO.Directory.Exists(dataFolderPath))
			{
				System.IO.Directory.CreateDirectory(dataFolderPath);
			}

			#endregion

			var locOptions = app.ApplicationServices.GetService<IOptions<RequestLocalizationOptions>>();
			app.UseRequestLocalization(locOptions.Value);

#if !DEBUG
			
			//app.UseHttpsRedirection();
#endif

			app.UseStaticFiles();

			app.UseAuthentication();

			//TODO server settings
			app.UseHangfireServer();
			app.UseHangfireDashboard("/hangfire", new DashboardOptions()
			{
				Authorization = new[] { new HangfireAuthorizationFilter() },
			});

			RecurringJob.AddOrUpdate<UpdateCompetitionDataTask>(t => t.Run(), "0 */10 * ? * *", TimeZoneInfo.Local);
			RecurringJob.AddOrUpdate<UpdateEventDataTask>(t => t.Run(), "0 */10 * ? * *", TimeZoneInfo.Local);
			RecurringJob.AddOrUpdate<ProgramDataPublisher>(t => t.Run("hangfire"), "0 * * * *", TimeZoneInfo.Local);
			RecurringJob.AddOrUpdate<ResultDataPublisher>(t => t.Run("hangfire"), "0 * * * *", TimeZoneInfo.Local);
			RecurringJob.AddOrUpdate<CheckFileAvailabilityTask>(t => t.Run(), "0 */30 * ? * *", TimeZoneInfo.Local);

			Log.Information("Hello World!");

			app.UseMvc(routes =>
			{
				routes.MapRoute(
					name: "default",
					template: "{controller=Home}/{action=Index}/{id?}");
			});
		}
	}
}
